// clang++ q61003206.cc -fsanitize=address ; and ./a.out

#include <array>
#include <iostream>
#include <vector>
#include <cassert>

namespace {
bool check_index(int idx) {
  if (idx >= 0 and idx < 9)
    return true;
  std::cerr << "Invalid Index: " << idx << std::endl;
  return false;
}
bool check_range(int val) {
  if (val >= 0 and val <= 9)  // TODO: I think that 0 should not be a valid value
    return true;
  std::cerr << "Value out of range: " << val << std::endl;
  return false;
}
} // namespace

struct Position {
  int digit{-1};
  std::vector<int> possible;
  bool isPermanent {false};
};

class Board {
public:
  explicit Board(std::array<int, 81> const &input);
  void print() const;

  std::array<int, 9> getRow(int row) const;
  std::array<int, 9> getColumn(int col) const;

  // make private
  void makePossibleList(int row, int col);
  void solve();

private:
  // maybe use an array of structures
  std::array<Position, 81> elements_;
  bool membership(const std::array<int, 9> &array, int digit) const;
};

Board::Board(std::array<int, 81> const &input) {
  elements_.fill(Position()); // TODO remove
  for (int i = 0; i < 81; i++) {
    Position temp;
    temp.digit = input[i];
    // possible is already initialized
    if (input[i] == 0) {
      temp.isPermanent = false;
    } else {
      // this position is set and cannot be changed
      temp.isPermanent = true;
    }

    elements_[i] = temp;
  }
}

void Board::print() const {
  for (int i = 0; i < elements_.size(); i++) {
    std::cout << "I: " << i << std::endl;
    // std::cout << elements_[i].digit << " ";
    if (i % 9 == 0) {std::cout << "\n";}
  }

  // for (int i = 0; i < 9; i++) {
  //   const auto arr = getRow(i);
  //   for (int j = 0; j < 9; j++) {
  //     assert(check_range(arr.at(j)));
  //     std::cout << arr.at(j) << " ";
  //   }
  //   std::cout << std::endl;
  // }
  // confirmed that it can print backwards
}

bool Board::membership(const std::array<int, 9> &arr, int digit) const {
  assert(check_range(digit));
  for (int i = 0; i < 9; i++) {
    assert(check_range(arr[i]));
    if (arr[i] == digit)
      return true;
  }
  return false;
}

void Board::makePossibleList(int row, int col) {
  assert(check_index(row));
  assert(check_index(col));
  Position temp = elements_[row * 9 + col];

  const auto tempRow = getRow(row);
  const auto tempColumn = getColumn(col);

  for (int i = 0; i < 9; i++) {
    if (membership(tempRow, i)) {
      continue;
    } else if (membership(tempColumn, i)) {
      continue;
    } else {
      temp.possible.push_back(i);
    }
  }

  // find the possible values for this position
}

std::array<int, 9> Board::getRow(int row) const {
  assert(check_index(row));
  std::array<int, 9> temp;
  temp.fill(-1); // TODO: remove
  for (int i = (row * 9); i < (9 * (row + 1)); i++) {
    assert(check_range(elements_[i].digit));
    temp[i - row * 9] = elements_[i].digit;
  }
  return temp;
}

std::array<int, 9> Board::getColumn(int col) const {
  assert(check_index(col));
  std::array<int, 9> temp;
  temp.fill(-1); // TODO: remove
  for (int i = 0; i < 81; i += 9) {
    assert(check_range(elements_[(i + col)].digit));
    temp[((i + 1) / 9)] = elements_[(i + col)].digit;
  }
  return temp;
}

void Board::solve() {
  for (int i = 0; i < 81; i++) {
    Position temp = elements_[i];

    if (temp.isPermanent) {
      continue;
    }
    else {
      int row = i / 9;
      int col = i % 9;
      makePossibleList(row, col);
      if (temp.possible.size() == 0) {
        break; // something messed up  TODO...
      }
      else {
        for (int i = 0; i < 9; i++) {
          std::cout << temp.possible[i] << " ";
        }
        std::cout << std::endl;

        // use the end of possible list value
        temp.digit = temp.possible.back(); // TODO: Assert that size() > 0!
      }
    }
  }
  // either the loop broke because something is wrong
  //     or the loop finished and the puzzle is solved
  print();
}

int main() {
  const std::array<int, 81> puzzle = {
      0, 2, 0, 0, 9, 0, 0, 6, 0, 0, 0, 9, 0, 5, 0, 3, 0, 2, 0, 8, 0,
      7, 0, 0, 0, 5, 0, 0, 6, 0, 0, 1, 0, 0, 0, 3, 0, 7, 0, 0, 3, 0,
      0, 9, 0, 9, 0, 0, 0, 6, 2, 0, 7, 0, 0, 3, 0, 0, 0, 1, 0, 2, 0,
      8, 0, 2, 0, 7, 0, 4, 0, 0, 0, 9, 0, 0, 8, 0, 0, 3, 0};
  Board x = Board(puzzle);
  // x.print();
  x.solve();
}
